from captcha.fields import CaptchaField
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import AuthenticationForm, PasswordResetForm
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from django_registration.forms import RegistrationFormUniqueEmail

User = get_user_model()

User.USERNAME_FIELD = 'email'
User.REQUIRED_FIELDS = ['username']


def get_user_by_email(email):
    try:
        user = User.objects.filter(email__iexact=email, is_active=True).get()
    except User.MultipleObjectsReturned:
        user = User.objects.filter(email__exact=email, is_active=True).get()
    except User.DoesNotExist:
        try:
            user = User.objects.filter(email__iexact=email).get()
        except User.MultipleObjectsReturned:
            user = User.objects.filter(email__exact=email).get()
        except User.DoesNotExist:
            try:
                user = User.objects.filter(username__exact=email, is_active=True).get()
            except User.DoesNotExist:
                user = User.objects.filter(username__exact=email).get()
    return user


class REPasswordResetForm(PasswordResetForm):
    captcha = CaptchaField()

    def clean_email(self):
        """
        Validate that the email is not already in use.
        """
        email = self.cleaned_data['email']
        user = get_user_by_email(email)
        if user:
            if not user.is_active:
                raise forms.ValidationError(
                    format_html(
                        _("User with this e-mail is not active. Please {} your account first."),
                        format_html(
                            "<a href='{activate_url}'>{activate_text}</a>",
                            activate_url=reverse('registration_resend_activation'),
                            activate_text=_("activate"),
                        ),
                    ),
                )

            # Generate random password, so that the e-mail is sent
            if not user.has_usable_password():
                user.password = User.objects.make_random_password()
                user.save()
        else:
            raise forms.ValidationError(
                "We don't have this e-mail in our system. Please check if you registered with this e-mail",
            )

        return email


class LoginForm(AuthenticationForm):
    def clean_username(self):
        username = self.cleaned_data['username']
        user = get_user_by_email(username)
        if not user:
            return username
        if not user.is_active:
            raise forms.ValidationError(
                format_html(
                    _("This account has not been activated yet. {}."),
                    format_html(
                        "<a href='{activate_url}'>{activate_text}</a>",
                        activate_url=reverse('registration_resend_activation'),
                        activate_text=_("Resend activation mail"),
                    ),
                ),
            )
        return user.email

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['username'].label = "Username or e-mail"


class FullUserRegistrationForm(RegistrationFormUniqueEmail):
    class Meta(RegistrationFormUniqueEmail.Meta):
        model = User
        fields = [
            'email',
            'password1',
            'password2',
        ]



from django.conf.urls import include
from django.contrib.auth.views import LoginView, PasswordResetView
from django.urls import path, reverse_lazy
from django.views.generic.base import TemplateView
from django_registration.backends.activation.views import RegistrationView

from .forms import LoginForm, REPasswordResetForm, FullUserRegistrationForm
from .views import NextURLActivationView

urlpatterns = [
    # The first url is here only because othewise they get overriden by the activation key url
    path(
        'activate/complete/',
        TemplateView.as_view(
            template_name='django_registration/activation_complete.html',
        ),
        name='django_registration_activation_complete',
    ),
    path(
        'accounts/register/',
        RegistrationView.as_view(
            form_class=FullUserRegistrationForm,
        ),
        name='django_registration_register',
    ),
    path(
        'activate/<str:activation_key>/',
        NextURLActivationView.as_view(),
        name='django_registration_activate',
    ),
    path(
        'login/',
        LoginView.as_view(
            form_class=LoginForm,
            template_name='registration/login.html',
        ),
        name='auth_login',
    ),
    path(
        'password/reset/',
        PasswordResetView.as_view(
            success_url=reverse_lazy('password_reset_done'),
            form_class=REPasswordResetForm,
            template_name='django_registration/password_reset_form.html',
        ),
        name='auth_password_reset',
    ),
    path('captcha/', include('captcha.urls')),
]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.conf.urls import include, url
from django.contrib import admin

urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^', include('django_registration_with_email.urls', namespace='django_registration_with_email')),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-registration-with-email
------------

Tests for `django-registration-with-email` models module.
"""

from django.test import TestCase

from django_registration_with_email import models


class TestDjango_registration_with_email(TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass
